<?php
class UsersController extends BackEndController {

	public $user;

	public function filters() {
		return CMap::mergeArray(parent::filters(), array(
			array('ext.RetrieveRecordFilter.RetrieveRecordFilter + view,update,delete', 'param' => 'id', 'attribute' => 'user', 'model' => 'User', 'message' => 'Invalid User!'),
		));
	}

	public function actionView($id) {
		$this->render('view', array(
			'user' => $this->user,
		));
	}

	/**
	 * Updates a particular model.
	 * If update is successful, the browser will be redirected to the 'view' page.
	 * @param integer $id the ID of the model to be updated
	 */
	public function actionUpdate() {
		$this->performAjaxValidation($this->user);

		if(isset($_POST[CHtml::modelName($this->user)])) {
			$this->user->attributes = $_POST[CHtml::modelName($this->user)];
			if($this->user->save())
				Yii::app()->user->setFlash('success', Yii::t('Users', 'User has been updated'));
				$this->redirect(array('view', 'id' => $this->user->id));
		}

		$this->render('update', array(
			'user' => $this->user,
		));
	}

	/**
	 * Deletes a particular model.
	 * If deletion is successful, the browser will be redirected to the 'admin' page.
	 * @param integer $id the ID of the model to be deleted
	 */
	public function actionDelete() {
		$this->user->delete();

		if(!isset($_GET['ajax']))
			$this->redirect(isset($_POST['returnUrl']) ? $_POST['returnUrl'] : array('index'));
	}

	/**
	 * Manages all models.
	 */
	public function actionIndex() {
		Yii::import('ext.SelectorWidget.*');
		$model = new User('search');
		$model->unsetAttributes();
		if(isset($_GET[CHtml::modelName(User::model())]))
			$model->attributes = $_GET[CHtml::modelName(User::model())];

		$page_size_labels = array(
			10 => Yii::t('Users', '{n} visitor on a page|{n} visitors on a page', 10),
			20 => Yii::t('Users', '{n} visitor on a page|{n} visitors on a page', 20),
			50 => Yii::t('Users', '{n} visitor on a page|{n} visitors on a page', 50),
			100 => Yii::t('Users', '{n} visitor on a page|{n} visitors on a page', 100),
		);
		$page_size_selector = new Selector($page_size_labels, 'page_size');
		$page_size_selector->defaultItem = 20;

		$this->render('index', array(
			'model' => $model,
			'page_size_selector' => $page_size_selector,
		));
	}

	/**
	 * Performs the AJAX validation.
	 * @param User $model the model to be validated
	 */
	protected function performAjaxValidation($model)
	{
		if(isset($_POST['ajax']) && $_POST['ajax']==='user-form')
		{
			echo CActiveForm::validate($model);
			Yii::app()->end();
		}
	}
}
